# frozen_string_literal: true

require 'stringio'
require 'apple_core/refinements/string'
require 'apple_core/refinements/hash'

module  Apiphobic
module  Requests
class   TransformJsonApi
  using ::AppleCore::Refinements::Hash
  using ::AppleCore::Refinements::String

  attr_accessor :request

  def initialize(request)
    self.request = request
  end

  def transform
    request['QUERY_STRING'] = query_string_with_underscored_parameters

    if has_content? && json?
      request['rack.input']     = request_body_with_underscored_json_keys
      request['CONTENT_LENGTH'] = content_length.to_s
    end

    request
  end

  private

  def has_content?
    request['CONTENT_LENGTH'].to_i.positive?
  end

  def json?
    request['CONTENT_TYPE'] =~ /json/
  end

  def content_length
    underscored_request_json.bytesize
  end

  def request_body_with_underscored_json_keys
    @request_body_with_underscored_json_keys ||= StringIO.new(underscored_request_json)
  end

  def query_string_with_underscored_parameters
    @query_string_with_underscored_parameters ||= begin
      return query_string unless query_string.respond_to?(:gsub)

      query_string.gsub(/(?<=\A|&|\?)[^=&]+/) do |parameter_name|
        unescaped_parameter_name   = CGI.unescape(parameter_name)
        underscored_parameter_name = unescaped_parameter_name.underscore

        CGI.escape(underscored_parameter_name)
      end
    end
  end

  def underscored_request_json
    @underscored_request_json ||= JSON.dump(underscored_request_hash)
  end

  def underscored_request_hash
    request_hash.deep_underscore_keys
  end

  def request_hash
    JSON.parse(request_body)
  end

  def request_body
    request['rack.input']&.read.to_s
  ensure
    request['rack.input']&.rewind
  end

  def query_string
    request['QUERY_STRING']
  end
end
end
end
